#!D:\Usr\Bin\Perl\bin\perl.exe

#***************************************************************************
#
# Perl SSI script
# Programmer: Greg Zwart (Gregory.Zwart@orst.edu)
# Created:    20000602
#
# Because some ISPs (namely mine) do not support server-side includes, this
# program attempts to simulate SSI using virtual includes.  This script will
# recursively traverse a "master" web directory and will search for virtual
# includes in all htm, html, shtm, and shtml files.  It will then replace
# the virtual include with the appropriate text/code from the include file.
# Finally, it will copy the entire web tree (with the parsed files) to a
# destination directory. 
#
# Usage: perlSSI.pl src_dir dest_dir virt_dir
#
#    src_dir   The location of the master web files that include the
#              SSI syntax
#    dest_dir  The destination directory into which the parsed file
#              tree should be copied
#    virt_dir  The "virtual" root of your web server file system.
#              For example, /usr/local/apache/htdocs
#
# NOTE: This script does very little (if any) error checking at this time,
#       so it really should not be used in a production environment.
#       However, it should be very handy for development purposes.
#
#**************************************************************************

use File::Copy;

#
# Declare the global variables
#
my $DEBUG = 0;  # Set to 1 to debug the script
my $RECURSE_DEPTH = 4;
my $VIRTUAL_ROOT;
my $SRC_ROOT;
my $DEST_ROOT;

#
# Declare the variables
#
my $dir;
my @files;
my $file;
my $curFile;
my $line;
my $ext;
my $ssiFile;
my $ssiLine;
my $tmpFile = "/ssi.tmp";
my $srcFile;
my $destFile;
my $destDir;
my $cnt;


# Read in the values from the command line, if a value is provided, replace
# all '\' with '/'.  If a value is not provided, then set a default. 
$SRC_ROOT  = $ARGV[0];
if($SRC_ROOT eq "") {
	print "ERROR: Source directory not specified!\n";
	exit;
} else {
	$SRC_ROOT  =~ s/\\/\//g;
}

$DEST_ROOT = $ARGV[1];
if($DEST_ROOT eq "") {
	print "ERROR: Source directory not specified!\n";
	exit;
} else {
	$DEST_ROOT =~ s/\\/\//g;
}

$VIRTUAL_ROOT = $ARGV[2];
if($VIRTUAL_ROOT eq "") {
	print "ERROR: Virtual root directory not specified!\n";
	exit;
} else {
	$VIRTUAL_ROOT =~ s/\\/\//g;
}

# Call the ssiReplace subroutine on the master web directory
$dir = $SRC_ROOT;
print "Copying from $dir to $DEST_ROOT\n" if $DEBUG;
ssiReplace($dir);

# Because SSI references can be included recursively, we'll need to 
# recursively replace those files.  The recurse depth variable will
# limit the number of times that we'll need to do an SSI check.
$DEST_ROOT =~ /(.*)\/(.*)/;
$VIRTUAL_ROOT = $1;
$SRC_ROOT = $DEST_ROOT;
$DEST_ROOT = $DEST_ROOT;
$cnt = 1;
while($cnt < $RECURSE_DEPTH) {
	ssiReplace($DEST_ROOT);
	$cnt++;
}

sub ssiReplace() {
	local($curDir) = $_[0];

	# Move into and read all the files contain in current directory
	chdir($curDir);
	opendir(THISDIR, $curDir);
	my @files = grep { !/^\./  } readdir(THISDIR);

	# Loop through all the files
	foreach $file ( @files ) {

		# Check to see if the curFile is a directory or a file
		$curFile = "$curDir/$file";
		print "FILE: $curFile\n" if $DEBUG;
		if(-X $curFile) {
		
			# For directories, we'll need to recursively call
			# the ssiReplace subroutine
			$curFile =~ /$SRC_ROOT\/(.*)/i;
			$destDir = "$DEST_ROOT/" . "$1";
			print "mkdir($destDir)\n" if $DEBUG;
			mkdir($destDir);
			ssiReplace($curFile);
		} else {

			$curFile =~ /\.([\w\d]*)$/;
			$ext = $1;
		
			# We only need to check for SSI references in html
			# htm, shtml, and shtm files
			if($ext=~/htm.$/i) {
				open(FILE, $curFile);
				open(TMP, ">$tmpFile");
				while($line = <FILE>) {
					if($line =~ /(.*)<\!--\#include\s+virtual="(.*)"\s*-->(.*)/) {
						print TMP "$1";
						$ssiFile = "$VIRTUAL_ROOT" . "$2";
						print "Replacing $line in $curFile with contents of $ssiFile\n" if $DEBUG;
						print "REPLACE: $curFile --> $line";
						open(SSI, $ssiFile);
						while($ssiLine = <SSI>) {
							print TMP "$ssiLine";
						}
						print TMP "$3";
					} else {
						print TMP "$line";
					}
				}
				close FILE;
				close TMP;

				$curFile =~ /$SRC_ROOT\/(.*)/i;
				$destFile = "$DEST_ROOT/$1";
				$srcFile = "$tmpFile";

				print "copy($srcFile, $destFile)\n" if $DEBUG;
				copy("$srcFile", "$destFile");
				unlink "$tmpFile";

			# We'll simply copy non-html files to the destination directory
			} else {
				$srcFile = "$curFile";
				$srcFile =~ /$SRC_ROOT\/(.*)/i;
				$destFile = "$DEST_ROOT/$1";
				print "copy($srcFile, $destFile)\n" if $DEBUG;
				copy("$srcFile", "$destFile");	
			}
		}
	}
	closedir THISDIR;
}
